import sqlite3
import os
from bs4 import BeautifulSoup
from datetime import datetime
#import markdown
import pypandoc
import re
import html
from contextlib import contextmanager
import sys
import traceback

#from sculblog.diego_flavored_markdown import DiegoFlavoredMarkdown, count_markdown_words
from sculblog.db import DbConn

#===============================
# Configuration and Utilities
#===============================

db_path = os.path.abspath(os.path.join('/', 'var', 'www', 'html', 'database', 'db.db'))
input_header_message = "what is the header of the page? "

def sanitize_filename(filename: str) -> str:
    return re.sub(r'[^\w\-_\. ]', '_', filename)

def remove_extension(filename):
    return os.path.splitext(filename)[0]

def custom_date():
    now = datetime.now()
    year, day, month_abbr = now.year, now.day, now.strftime('%b')
    month = 'July' if month_abbr == 'Jul' else month_abbr
    return f"{year} {month} {day:02d}"

#===============================
# HTML and Content Management
#===============================


def write_page_html(file_path: str):
    try:
        html = pypandoc.convert_file(
            file_path,
            to='html',
            format='markdown-markdown_in_html_blocks+attributes+fenced_divs',
            extra_args=['--mathml']
        )
    except Exception as e:
        print(f"Error in write_page_html: {str(e)}")
        raise
    return html

def write_preview_html(html_content: str, char_len: int):
    try:
        # Parse HTML content
        soup = BeautifulSoup(html_content, 'html.parser')
        
        for s_tag in soup.find_all(['style', 'iframe']):
            s_tag.replace_with("")

        # Convert link tags to spans
        for a_tag in soup.find_all('a'):
            span_tag = soup.new_tag('span', **{'class': 'false-external-link'})
            span_tag.string = a_tag.get_text()
            a_tag.replace_with(span_tag)
        
        # Convert header tags to spans
        for h_tag in soup.find_all(['h1', 'h2', 'h3', 'h4', 'h5', 'h6']):
            span_tag = soup.new_tag('span', **{'class': 'preview-header'})
            span_tag.string = h_tag.get_text()
            h_tag.replace_with(span_tag)
        
        # Unwrap any tags that aren't standard text formatting elements
        for tag in soup.find_all():
            if tag.name not in ['span', 'sup', 'sub', 'em', 'strong', 'i', 'b',
                                's', 'strike', 'del',  # Strikethrough variations
                                'u',                   # Underline
                                'mark',                # Highlight/marking
                                'small',               # Smaller text
                                'q', 'cite',           # Quotations
                                'code', 'kbd',         # Code and keyboard input
                                'abbr',                # Abbreviations
                            ]:           
                tag.unwrap()
        
        # Get all text content to check length
        text_content = soup.get_text()
        
        # If text is longer than char_len, truncate
        if len(text_content) > char_len:
            # Find all text nodes and accumulate up to char_len
            text_nodes = []
            current_count = 0
            
            for element in soup.descendants:
                if isinstance(element, str) and current_count < char_len:
                    if current_count + len(element) <= char_len:
                        text_nodes.append((element, element))
                        current_count += len(element)
                    else:
                        # Truncate this text node
                        truncated = element[:char_len - current_count] + "..."
                        text_nodes.append((element, truncated))
                        current_count = char_len
                        break
            
            # Replace text nodes with truncated versions
            for original, truncated in text_nodes:
                if original != truncated:
                    original.replace_with(truncated)
        
        # Get final HTML with all tags properly closed
        clean_html = ''.join(str(tag) for tag in soup.contents)
        return clean_html
        
    except Exception as e:
        print(f"Error in write_preview_html: {str(e)}")
        print("Traceback:")
        traceback.print_exc()
        raise

#===============================
# Main Functions
#===============================

def post_updater(func):
    def wrapper(category, file_path): 
        try:
            is_draft = func(category, file_path)

            db = DbConn(db_path)
                      
            page_html = write_page_html(file_path)
            
            post_data = {
                "text": page_html,
                "preview_html": write_preview_html(page_html, 1500),
                "word_count": count_markdown_words(markdown_content), 
            }
            
            file_name = os.path.splitext(file_path)[0]
            
            if not db.value_in_column(category, "file_name", file_name):
                post_data["header"] = input(input_header_message)
                post_data["file_name"] = file_name
                post_data["date_splash"] = custom_date()
                
                # draft 0 is for preview. everything else is for regular drafts
                for x in range(0,2):
                    post_data["draft"] = x
                    if db.insert_row(category, post_data):
                        pass
                    else:
                        raise Error("Failed to insert data.")
                    print(f"Successfully inserted data for post {file_name}")    
                
            else:
                # db.update_row(category, "file_name", file_name, post_data)
                set_clause = ", ".join([f"{col} = ?" for col in post_data.keys()])

                # if it is draft, then it makes a copy and increments the draft counter. if it is preview, then it just updates the where draft is zero
                if is_draft: 

                    columns = ', '.join(post_data.keys()) + ', file_name, draft'
                    placeholders = ', '.join(['?' for _ in post_data])
                    subquery_draft = f"(SELECT COALESCE(MAX(draft), 0) + 1 FROM {category} WHERE file_name = ?)"
                    where_clause = f"WHERE file_name = ? AND draft = (SELECT MAX(draft) FROM {category} WHERE file_name = ?)"
                    values = list(post_data.values()) + [file_name, file_name, file_name, file_name]

                    query = f"""
                        INSERT INTO {category} ({columns})
                        SELECT {placeholders}, ?, {subquery_draft}
                        FROM {category}
                        {where_clause}
                        LIMIT 1
                    """

                else:
                    values = list(post_data.values()) + [file_name]
                    where_clause = "file_name = ? AND draft = 0"

                    query = f"""
                        UPDATE {category}
                        SET {set_clause}
                        WHERE {where_clause} 
                    """

                db.execute_query(query, values)
                
        except Exception as e:
            print(f"An error of Exception type {type(e).__name__} occurred: {e}")   

    return wrapper

@post_updater
def preview(category, file_path):
    return False

@post_updater
def draft(category, file_path):
    return True

def db_updater(func):
    def wrapper(category, file_path, *args, **kwargs):
        file_name = remove_extension(file_path)
        field, value = func(*args, **kwargs)
        DbConn(db_path).update_row(category, "file_name", file_name, {field: value})
    return wrapper

@db_updater
def hide():
    return "hide", "0"

@db_updater
def unhide():
    return "hide", ""

@db_updater
def date():
    date_splash = input("What is the date of the page? ")
    return "date_splash", date_splash

@db_updater
def header():
    header = input(input_header_message)
    return "header", header

def main():
    if len(sys.argv) < 3:
        print("Usage: sculblog <command> <category> <postname> \n\t  commands: preview, draft, hide, unhide, date, header")
        sys.exit(1)
        
    command, category, file_path = sys.argv[1], sys.argv[2], sys.argv[3]
    
    if command == "preview":
        preview(category, file_path)
    elif command == "draft":
        draft(category, file_path)

    elif command == "hide":
        hide(category, file_path)
    elif command == "unhide":
        unhide(category, file_path)

    elif command == "date":
        date(category, file_path)
    elif command == "header":
        header(category, file_path)

    else:
        print('Invalid command. Command must be "preview", "draft", "hide", "unhide", "date", or "header"')
        sys.exit(1)

if __name__ == "__main__":
    main()
